
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                        Infos gnrales                                %%
%Programme dvelopp en 2015 au Laboratoire CAR, quipe Atomes Froids     %
%118, route de Narbonne - 31062 Toulouse Cedex 09 - France                %
%Pour plus d'information contactez                                        %
%Ala Fortun : aelafortun@hotmail.com   
%David Gury-Odelin : dgo@irsamc.ups-tlse.fr
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%        Traitement de l'amplitude d'oscillation - dphasage 45        %%
%                                                                         %
%                                                                         %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%              fichiers supplmentaires ncssaires :                   %%
%   Fichiers gnrs par le programme 'traitement_globalplot :            %
%  imNom_DataIntegClean.txt  : Film intgr et dont le fond thermique    % 
%                              t retir                                 %
%                imNom_X.txt : Base spatiale horizontale correspondant   %
%                              l'exprience                               %
%                imNom_T.txt : Base temporelle correspondant             %
%                              l'exprience                               %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all
close all
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                Dfinition des paramtres                              %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%  Chargement des donnes mises en formes par globalplot.m
%  Nous chargeons DataIntegClean, X et T
%  Il faut renseigner ImNom 

ImNom='Oscillation_2p8El_';
Dataclean=load(strcat(ImNom,'DataIntegClean.txt'));
Xgrid=load(strcat(ImNom,'X.txt'));
Tgrid=load(strcat(ImNom,'T.txt'));
Ntemps=length(Tgrid);  %% Nombre d'images
Tgridflag=Tgrid;  %  Base temporelle  laquelle nous retirerons les points 
                  %  le programme n'arrive pas a ajuster
PixelSize=2.68;  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                        Dfinition du cadre                            %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Cadregauche et Cadredroit dfinissent la partie gauche de l'image et
% la partie doir. Ces deux parties doivent inclure la moite de l'ordre 
% d'interfrence 0 et ne pas inclure trop de fond (il faut donc couper le dbut et la fin de
% le dbut et la fin de chaque image)


figure(1)
imagesc(Dataclean)
title('Dfinition du cadre')

G=input('Limite gauche : ');
M=input('Centre de l ordre 0 : ');
D=input('Limite droite : ');
Cadregauche=[G,M];
Cadredroit=[M,D];


l=1; % Initialisation du compteur qui prendra en compte les points que le 
     % programme n'arrive pas a ajuster

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%   Dtermination du critre de basculement entre 1 et 2 gaussiennes    %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Si la vitesse moyenne est nulle (i.e. si la gaussienne est centre, nous
% voulons ajuster l'enveloppe avec une seule gausienne. Sinon, nous voulons
% utiliser 2 gaussiennes correspondant respectivement  la partie gauche et
% droite de l'image. Pour cela, le critre de basculement est la variable
% varbar.


for i=1:Ntemps
    
    Xg=Xgrid(Cadregauche(1):Cadregauche(2));
    Xd=Xgrid(Cadredroit(1):Cadredroit(2));
    Dg=Dataclean(i,Cadregauche(1):Cadregauche(2));
    Dd=Dataclean(i,Cadredroit(1):Cadredroit(2));
    
    % Initialisation du centre de gaussienne de droite (resp. gauche) par 
    % le barycentre de la position des atomes ayant une vitesse positive
    % (resp. negative) sur le cadre droit (resp. gauche) dfini plus haut.
    
    Barg(i)=sum(Xg.*Dg)/sum(Dg);  
    Bard(i)=sum(Xd.*Dd)/sum(Dd);
    bartot(i)=sum(Xgrid.*Dataclean(i,:));
    
    %varbar nous sert  dterminer si il faut 1 ou 2 gaussienne pour fiter
    %l'enveloppe.
    varbar(i)=(2*(Barg(i)-Bard(i))/(Barg(i)+Bard(i)))^2;
end

figure(1)
plot(varbar)
title('Dtermination du critre de basculement')

Bascul=input('Critre de basculement : ');


     
for i=1:Ntemps  %%dbut de la boucle sur le temps
    Tgrid(i)
  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
    %% Dtermination du nombre d'ondre d'interfrence  ajuster %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    
    [pks,loc]=findpeaks(Dataclean(i,:),'MinPeakDistance',20,'MinPeakHeight',0.0005);
    [yeah,Nyeah]=max(pks); %ordre d'interfrence le plus grand
    Npeak=length(loc);
    Npeakgauche=0;
    Npeakdroit=0;
    
    if varbar(i)>Bascul %critre empirique
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%          Il faut 2 gaussiennes         %%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        Nbgauss(i)=2; 


        init=[0.01,0.01]; %initialisation de l'amplitude des gausiennes d'enveloppe
        
        %%%%%Dtermination du nombre d'ordre d'interfrence de chaque cot
        for j=1:Npeak
            if loc(j)<M-30
                Npeakgauche=Npeakgauche+1;
            elseif loc(j)>M+30
                Npeakdroit=Npeakdroit+1;
            end
        end
        
        
        %%%%%%%%Construction de la fonction qui nous servira a ajuster et
        %%%%%%%%de l'initialisation de chacun des centres
        for n=1:Npeak
            if n==1
                oscilfit=strcat('(Ag*exp(-(p-Bg)^2/Cg)+Ad*exp(-(p-Bd)^2/Cd))*(exp(-(p-B',num2str(n),')^2/D)');
                init(n+2)=2.68*loc(n);
            elseif n==Npeak
                oscilfit=strcat(oscilfit,'+exp(-(p-B',num2str(n),')^2/D))');
                init(n+2)=2.68*loc(n);
            else
                oscilfit=strcat(oscilfit,'+exp(-(p-B',num2str(n),')^2/D)');
                init(n+2)=2.68*loc(n);
            end
        end
        
        %%%initialisation du reste des variables
        init(Npeak+3)=Bard(i);
        init(Npeak+4)=Barg(i);
        init(Npeak+5)=Npeakdroit*4000;
        init(Npeak+6)=Npeakgauche*4000;
        init(Npeak+7)=900;

        
        %%%Ajustement
        oscilfittype=fittype(oscilfit,'independent','p');
        [fitoscil,gofoscil,paramfit]=fit(Xgrid',Dataclean(i,:)',oscilfittype,'StartPoint',init,'Algorithm','Levenberg-Marquardt');
        
        flag=paramfit.exitflag; 
        if (flag>0) && (gofoscil.rsquare>0.9) %%%Si l'ajustement n'a pas 
                                              %%%converg ou si la qualit
                                              %%%de l'ajustement n'est pas
                                              %%%suffisant, on rejette le
                                              %%%point
                                              
            % Enregistrement des rsultats de l'ajustement
            
            pos(l,1)=fitoscil.Bg;
            pos(l,2)=fitoscil.Bd;
            goftot(i)=gofoscil;
            A(l,1)=fitoscil.Ag;
            A(l,2)=fitoscil.Ad;
            
            if fitoscil.Ag>fitoscil.Ad
                posmax(l)=fitoscil.Bg;
            else
                posmax(l)=fitoscil.Bd;
            end
            l=l+1;
        else
            Tgridflag(l)=[];
        end
            
    else
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%          Il faut 1 gaussienne          %%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
        Nbgauss(i)=1;
    
        %initialisation de l'amplitude et du centre
        init=[0.03,loc(Nyeah)*2.68];
        
        
        %%%%%%%%Construction de la fonction qui nous servira a ajuster et
        %%%%%%%%de l'initialisation de chacun des centres
    
        for n=1:Npeak
            if n==1
                oscilfit=strcat('(A*exp(-(p-B)^2/C))*(exp(-(p-B',num2str(n),')^2/D)');
                init(n+2)=2.68*loc(n);
            elseif n==Npeak
                oscilfit=strcat(oscilfit,'+exp(-(p-B',num2str(n),')^2/D))');
                init(n+2)=2.68*loc(n);
            else
                oscilfit=strcat(oscilfit,'+exp(-(p-B',num2str(n),')^2/D)');
                init(n+2)=2.68*loc(n);
            end
        end
        
        %initialisation du reste des variables
        
        init(Npeak+3)=Npeak*3000;
        init(Npeak+4)=900;

        %Ajustement
        oscilfittype=fittype(oscilfit,'independent','p');
        [fitoscil,gofoscil,paramfit]=fit(Xgrid',Dataclean(i,:)',oscilfittype,'StartPoint',init,'Algorithm','Levenberg-Marquardt');
        flag=paramfit.exitflag;
        
        if (flag>0) && (gofoscil.rsquare>0.9) %Mme critre de rejection
            %Enregistrement des rsultats de l'ajustement
            pos(l,1)=fitoscil.B;
            pos(l,2)=fitoscil.B;
            goftot(l)=gofoscil;
            confrs(l)=gofoscil.rsquare;
            A(l,1)=fitoscil.A;
            A(l,2)=fitoscil.A;
            posmax(l)=fitoscil.B;
            l=l+1;
        else
            Tgridflag(l)=[];
        end



        
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%               Affichage des rsults des ajustements                   %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NB : si trop peut de points sont retenus, cela vient  priori d'un
% mauvais choix de critre de basculement ou d'un critre trop contraignant
% sur la qualit des ajustements.

figure(1)
imagesc(Xgrid,Tgrid,Dataclean)
hold on
plot(pos,Tgridflag,'g-o')
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%                   Ajustement de l'oscilation                          %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%Il faut choisir sur quel plage on veut faire l'ajustement (cela dpend de
%la qualit des donnes donc ca peut demander quelques itrations)

Tgridfit=Tgridflag(1:l-1);
posgfit=pos(1:l-1,1);
posdfit=pos(1:l-1,2);
posmaxfit=posmax(1:l-1);

%Ajustement de l'oscillation  gauche, de celle  droite, et de celle
%correpondant au paquet dont l'amplitude est la plus grande

sinposfit='A*sin(2*pi*t/B+C)+D';
sinposfittype=fittype(sinposfit,'independent','t');
fitposd=fit(Tgridfit',posdfit,sinposfittype,'StartPoint',[150,60,0.3,800],'Algorithm','Levenberg-Marquardt');
fitposg=fit(Tgridfit',posgfit,sinposfittype,'StartPoint',[150,60,1.3,600],'Algorithm','Levenberg-Marquardt');
fitposmax=fit(Tgridfit',posmaxfit',sinposfittype,'StartPoint',[120,82,0.3,705],'Algorithm','Levenberg-Marquardt');

% Affichage des rsultats des ajustements. Suivant le cas (si il y a
% sparation des paquets en deux ou non, il faut choisir les rsultats qui
% semblent les plus cohrents. 
figure(2)
plot(fitposd,'b',Tgridfit,posdfit,'b--+')
hold on
plot(fitposg,'r',Tgridfit,posgfit,'r--+')
hold on
plot(fitposmax,'k',Tgridfit,posmaxfit,'k--+')


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%        Dtermination de la profondeur et des barres d'erreurs         %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


hbar=1.05*10^(-34);
m=1.45*10^(-25);
d=532*10^(-9);
vl=2*pi*hbar/(m*d);
El=m*vl^2/2;
xmaxd=1.1*2*fitposd.A;
confd=confint(fitposd);
dxmaxd=(confd(2,1)-confd(1,1))/2;
vmaxd=xmaxd*10^(-6)/(20*10^(-3));
dvmaxd=dxmaxd*vmaxd/xmaxd;
sd=m*vmaxd^2/El
dsd=abs(dvmaxd*sd/vmaxd)


xmaxg=1.1*2*fitposg.A;
confg=confint(fitposg);
dxmaxg=(confg(2,1)-confg(1,1))/2;
vmaxg=xmaxg*10^(-6)/(20*10^(-3));
dvmaxg=dxmaxg*vmaxg/xmaxg;
sg=m*vmaxg^2/El
dsg=abs(dvmaxg*sg/vmaxg)

xmax=1.1*fitposmax.A;
conf=confint(fitposmax);
dxmax=(conf(2,1)-conf(1,1))/2;
vmax=xmax*10^(-6)/(20*10^(-3));
dvmax=dxmax*vmax/xmax;
s=m*vmax^2/El
ds=abs(dvmax*s/vmax)


save('profondeurg.txt','sg','-ascii')
save('profondeurd.txt','sd','-ascii')
save('profondeur.txt','s','-ascii')
% figure(3)
% plot(Tgrid,confrs,'b')
    